@@ -51,8 +51,6 @@ static void validateConnectbyTupleDesc(TupleDesc tupdesc, bool show_branch, bool
 static bool compatCrosstabTupleDescs(TupleDesc tupdesc1, TupleDesc tupdesc2);
 static bool compatConnectbyTupleDescs(TupleDesc tupdesc1, TupleDesc tupdesc2);
 static void get_normal_pair(float8 *x1, float8 *x2);
-static TupleDesc make_crosstab_tupledesc(TupleDesc spi_tupdesc,
-						int num_categories);
 static Tuplestorestate *connectby(char *relname,
 		  char *key_fld,
 		  char *parent_key_fld,
@@ -156,10 +154,6 @@ do { \
 	snprintf(key, MAX_CATNAME_LEN - 1, "%s", CATDESC->catname); \
 	hentry = (crosstab_HashEnt*) hash_search(crosstab_HashTable, \
 										 key, HASH_ENTER, &found); \
-	if (hentry == NULL) \
-		ereport(ERROR, \
-				(errcode(ERRCODE_OUT_OF_MEMORY), \
-				 errmsg("out of memory"))); \
 	if (found) \
 		ereport(ERROR, \
 				(errcode(ERRCODE_DUPLICATE_OBJECT), \
@@ -202,8 +196,7 @@ normal_rand(PG_FUNCTION_ARGS)
 		funcctx = SRF_FIRSTCALL_INIT();
 
 		/*
-		 * switch to memory context appropriate for multiple function
-		 * calls
+		 * switch to memory context appropriate for multiple function calls
 		 */
 		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
 
@@ -214,10 +207,10 @@ normal_rand(PG_FUNCTION_ARGS)
 		fctx = (normal_rand_fctx *) palloc(sizeof(normal_rand_fctx));
 
 		/*
-		 * Use fctx to keep track of upper and lower bounds from call to
-		 * call. It will also be used to carry over the spare value we get
-		 * from the Box-Muller algorithm so that we only actually
-		 * calculate a new value every other call.
+		 * Use fctx to keep track of upper and lower bounds from call to call.
+		 * It will also be used to carry over the spare value we get from the
+		 * Box-Muller algorithm so that we only actually calculate a new value
+		 * every other call.
 		 */
 		fctx->mean = PG_GETARG_FLOAT8(1);
 		fctx->stddev = PG_GETARG_FLOAT8(2);
@@ -272,7 +265,7 @@ normal_rand(PG_FUNCTION_ARGS)
 		SRF_RETURN_NEXT(funcctx, Float8GetDatum(result));
 	}
 	else
-	/* do when there is no more left */
+		/* do when there is no more left */
 		SRF_RETURN_DONE(funcctx);
 }
 
@@ -348,12 +341,14 @@ get_normal_pair(float8 *x1, float8 *x2)
  * NOTES:
  * 1. SQL result must be ordered by 1,2.
  * 2. The number of values columns depends on the tuple description
- *	  of the function's declared return type.
- * 2. Missing values (i.e. not enough adjacent rows of same rowid to
+ *	  of the function's declared return type.  The return type's columns
+ *	  must match the datatypes of the SQL query's result.  The datatype
+ *	  of the category column can be anything, however.
+ * 3. Missing values (i.e. not enough adjacent rows of same rowid to
  *	  fill the number of result values columns) are filled in with nulls.
- * 3. Extra values (i.e. too many adjacent rows of same rowid to fill
+ * 4. Extra values (i.e. too many adjacent rows of same rowid to fill
  *	  the number of result values columns) are skipped.
- * 4. Rows with all nulls in the values columns are skipped.
+ * 5. Rows with all nulls in the values columns are skipped.
  */
 PG_FUNCTION_INFO_V1(crosstab);
 Datum
@@ -377,10 +372,7 @@ crosstab(PG_FUNCTION_ARGS)
 	if (SRF_IS_FIRSTCALL())
 	{
 		char	   *sql = GET_STR(PG_GETARG_TEXT_P(0));
-		Oid			funcid = fcinfo->flinfo->fn_oid;
-		Oid			functypeid;
-		char		functyptype;
-		TupleDesc	tupdesc = NULL;
+		TupleDesc	tupdesc;
 		int			ret;
 		int			proc;
 
@@ -388,8 +380,7 @@ crosstab(PG_FUNCTION_ARGS)
 		funcctx = SRF_FIRSTCALL_INIT();
 
 		/*
-		 * switch to memory context appropriate for multiple function
-		 * calls
+		 * switch to memory context appropriate for multiple function calls
 		 */
 		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
 
@@ -408,20 +399,23 @@ crosstab(PG_FUNCTION_ARGS)
 			spi_tuptable = SPI_tuptable;
 			spi_tupdesc = spi_tuptable->tupdesc;
 
-			/*
+			/*----------
 			 * The provided SQL query must always return three columns.
 			 *
-			 * 1. rowname	the label or identifier for each row in the final
-			 * result 2. category  the label or identifier for each column
-			 * in the final result 3. values	the value for each column
-			 * in the final result
+			 * 1. rowname
+			 *	the label or identifier for each row in the final result
+			 * 2. category
+			 *	the label or identifier for each column in the final result
+			 * 3. values
+			 *	the value for each column in the final result
+			 *----------
 			 */
 			if (spi_tupdesc->natts != 3)
 				ereport(ERROR,
 						(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 						 errmsg("invalid source data SQL statement"),
-						 errdetail("The provided SQL must return 3 " \
-							 " columns; rowid, category, and values.")));
+						 errdetail("The provided SQL must return 3 "
+								   "columns: rowid, category, and values.")));
 		}
 		else
 		{
@@ -433,39 +427,31 @@ crosstab(PG_FUNCTION_ARGS)
 		/* SPI switches context on us, so reset it */
 		MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
 
-		/* get the typeid that represents our return type */
-		functypeid = get_func_rettype(funcid);
-
-		/* check typtype to see if we have a predetermined return type */
-		functyptype = get_typtype(functypeid);
-
-		if (functyptype == 'c')
+		/* get a tuple descriptor for our result type */
+		switch (get_call_result_type(fcinfo, NULL, &tupdesc))
 		{
-			/* Build a tuple description for a named composite type */
-			tupdesc = TypeGetTupleDesc(functypeid, NIL);
-		}
-		else if (functypeid == RECORDOID)
-		{
-			if (fcinfo->nargs != 2)
+			case TYPEFUNC_COMPOSITE:
+				/* success */
+				break;
+			case TYPEFUNC_RECORD:
+				/* failed to determine actual type of RECORD */
 				ereport(ERROR,
-						(errcode(ERRCODE_SYNTAX_ERROR),
-						 errmsg("wrong number of arguments")));
-			else
-			{
-				int			num_categories = PG_GETARG_INT32(1);
-
-				tupdesc = make_crosstab_tupledesc(spi_tupdesc, num_categories);
-			}
+						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+						 errmsg("function returning record called in context "
+								"that cannot accept type record")));
+				break;
+			default:
+				/* result type isn't composite */
+				elog(ERROR, "return type must be a row type");
+				break;
 		}
-		else
-			ereport(ERROR,
-					(errcode(ERRCODE_SYNTAX_ERROR),
-					 errmsg("return type must be a row type")));
+
+		/* make sure we have a persistent copy of the tupdesc */
+		tupdesc = CreateTupleDescCopy(tupdesc);
 
 		/*
-		 * Check that return tupdesc is compatible with the one we got
-		 * from ret_relname, at least based on number and type of
-		 * attributes
+		 * Check that return tupdesc is compatible with the data we got from
+		 * SPI, at least based on number and type of attributes
 		 */
 		if (!compatCrosstabTupleDescs(tupdesc, spi_tupdesc))
 			ereport(ERROR,
@@ -474,8 +460,8 @@ crosstab(PG_FUNCTION_ARGS)
 							"incompatible")));
 
 		/*
-		 * Generate attribute metadata needed later to produce tuples from
-		 * raw C strings
+		 * Generate attribute metadata needed later to produce tuples from raw
+		 * C strings
 		 */
 		attinmeta = TupleDescGetAttInMetadata(tupdesc);
 		funcctx->attinmeta = attinmeta;
@@ -703,8 +689,8 @@ crosstab(PG_FUNCTION_ARGS)
  * 1. SQL result must be ordered by 1.
  * 2. The number of values columns depends on the tuple description
  *	  of the function's declared return type.
- * 2. Missing values (i.e. missing category) are filled in with nulls.
- * 3. Extra values (i.e. not in category results) are skipped.
+ * 3. Missing values (i.e. missing category) are filled in with nulls.
+ * 4. Extra values (i.e. not in category results) are skipped.
  */
 PG_FUNCTION_INFO_V1(crosstab_hash);
 Datum
@@ -719,9 +705,13 @@ crosstab_hash(PG_FUNCTION_ARGS)
 	int			num_categories;
 
 	/* check to see if caller supports us returning a tuplestore */
-	if (!rsinfo || !(rsinfo->allowedModes & SFRM_Materialize))
+	if (rsinfo == NULL || !IsA(rsinfo, ReturnSetInfo))
 		ereport(ERROR,
-				(errcode(ERRCODE_SYNTAX_ERROR),
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("set-valued function called in context that cannot accept a set")));
+	if (!(rsinfo->allowedModes & SFRM_Materialize))
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				 errmsg("materialize mode required, but it is not " \
 						"allowed in this context")));
 
@@ -758,10 +748,10 @@ crosstab_hash(PG_FUNCTION_ARGS)
 
 	/*
 	 * SFRM_Materialize mode expects us to return a NULL Datum. The actual
-	 * tuples are in our tuplestore and passed back through
-	 * rsinfo->setResult. rsinfo->setDesc is set to the tuple description
-	 * that we actually used to build our tuples with, so the caller can
-	 * verify we did what it was expecting.
+	 * tuples are in our tuplestore and passed back through rsinfo->setResult.
+	 * rsinfo->setDesc is set to the tuple description that we actually used
+	 * to build our tuples with, so the caller can verify we did what it was
+	 * expecting.
 	 */
 	rsinfo->setDesc = tupdesc;
 	MemoryContextSwitchTo(oldcontext);
@@ -786,8 +776,8 @@ load_categories_hash(char *cats_sql, MemoryContext per_query_ctx)
 	ctl.entrysize = sizeof(crosstab_HashEnt);
 
 	/*
-	 * use INIT_CATS, defined above as a guess of how many hash table
-	 * entries to create, initially
+	 * use INIT_CATS, defined above as a guess of how many hash table entries
+	 * to create, initially
 	 */
 	crosstab_HashTable = hash_create("crosstab hash", INIT_CATS, &ctl, HASH_ELEM);
 
@@ -808,8 +798,8 @@ load_categories_hash(char *cats_sql, MemoryContext per_query_ctx)
 		int			i;
 
 		/*
-		 * The provided categories SQL query must always return one
-		 * column: category - the label or identifier for each column
+		 * The provided categories SQL query must always return one column:
+		 * category - the label or identifier for each column
 		 */
 		if (spi_tupdesc->natts != 1)
 			ereport(ERROR,
@@ -901,26 +891,24 @@ get_crosstab_tuplestore(char *sql,
 		}
 
 		/*
-		 * The provided SQL query must always return at least three
-		 * columns:
+		 * The provided SQL query must always return at least three columns:
 		 *
 		 * 1. rowname	the label for each row - column 1 in the final result
-		 * 2. category	the label for each value-column in the final
-		 * result 3. value	   the values used to populate the
-		 * value-columns
+		 * 2. category	the label for each value-column in the final result 3.
+		 * value	 the values used to populate the value-columns
 		 *
 		 * If there are more than three columns, the last two are taken as
-		 * "category" and "values". The first column is taken as
-		 * "rowname". Additional columns (2 thru N-2) are assumed the same
-		 * for the same "rowname", and are copied into the result tuple
-		 * from the first time we encounter a particular rowname.
+		 * "category" and "values". The first column is taken as "rowname".
+		 * Additional columns (2 thru N-2) are assumed the same for the same
+		 * "rowname", and are copied into the result tuple from the first time
+		 * we encounter a particular rowname.
 		 */
 		if (ncols < 3)
 			ereport(ERROR,
 					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 					 errmsg("invalid source data SQL statement"),
 					 errdetail("The provided SQL must return 3 " \
-							 " columns; rowid, category, and values.")));
+							   " columns; rowid, category, and values.")));
 
 		result_ncols = (ncols - 2) + num_categories;
 
@@ -931,7 +919,7 @@ get_crosstab_tuplestore(char *sql,
 					 errmsg("invalid return type"),
 					 errdetail("query-specified return " \
 							   "tuple has %d columns but crosstab " \
-						   "returns %d", tupdesc->natts, result_ncols)));
+							   "returns %d", tupdesc->natts, result_ncols)));
 
 		/* allocate space */
 		values = (char **) palloc(result_ncols * sizeof(char *));
@@ -963,16 +951,19 @@ get_crosstab_tuplestore(char *sql,
 				 */
 				if (!firstpass)
 				{
+					/*
+					 * switch to appropriate context while storing the tuple
+					 */
+					SPIcontext = MemoryContextSwitchTo(per_query_ctx);
+
 					/* rowid changed, flush the previous output row */
 					tuple = BuildTupleFromCStrings(attinmeta, values);
-
-					/* switch to appropriate context while storing the tuple */
-					SPIcontext = MemoryContextSwitchTo(per_query_ctx);
 					tuplestore_puttuple(tupstore, tuple);
-					MemoryContextSwitchTo(SPIcontext);
-
 					for (j = 0; j < result_ncols; j++)
 						xpfree(values[j]);
+
+					/* now reset the context */
+					MemoryContextSwitchTo(SPIcontext);
 				}
 
 				values[0] = rowid;
@@ -1004,11 +995,11 @@ get_crosstab_tuplestore(char *sql,
 
 		/* flush the last output row */
 		tuple = BuildTupleFromCStrings(attinmeta, values);
-
-		/* switch to appropriate context while storing the tuple */
-		SPIcontext = MemoryContextSwitchTo(per_query_ctx);
 		tuplestore_puttuple(tupstore, tuple);
+
+		/* now reset the context */
 		MemoryContextSwitchTo(SPIcontext);
+
 	}
 
 	if (SPI_finish() != SPI_OK_FINISH)
@@ -1077,9 +1068,13 @@ connectby_text(PG_FUNCTION_ARGS)
 	MemoryContext oldcontext;
 
 	/* check to see if caller supports us returning a tuplestore */
-	if (!rsinfo || !(rsinfo->allowedModes & SFRM_Materialize))
+	if (rsinfo == NULL || !IsA(rsinfo, ReturnSetInfo))
 		ereport(ERROR,
-				(errcode(ERRCODE_SYNTAX_ERROR),
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("set-valued function called in context that cannot accept a set")));
+	if (!(rsinfo->allowedModes & SFRM_Materialize))
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				 errmsg("materialize mode required, but it is not " \
 						"allowed in this context")));
 
@@ -1104,13 +1099,6 @@ connectby_text(PG_FUNCTION_ARGS)
 	/* OK, use it then */
 	attinmeta = TupleDescGetAttInMetadata(tupdesc);
 
-	/* check to see if caller supports us returning a tuplestore */
-	if (!rsinfo || !(rsinfo->allowedModes & SFRM_Materialize))
-		ereport(ERROR,
-				(errcode(ERRCODE_SYNTAX_ERROR),
-				 errmsg("materialize mode required, but it is not " \
-						"allowed in this context")));
-
 	/* OK, go to work */
 	rsinfo->returnMode = SFRM_Materialize;
 	rsinfo->setResult = connectby(relname,
@@ -1130,10 +1118,10 @@ connectby_text(PG_FUNCTION_ARGS)
 
 	/*
 	 * SFRM_Materialize mode expects us to return a NULL Datum. The actual
-	 * tuples are in our tuplestore and passed back through
-	 * rsinfo->setResult. rsinfo->setDesc is set to the tuple description
-	 * that we actually used to build our tuples with, so the caller can
-	 * verify we did what it was expecting.
+	 * tuples are in our tuplestore and passed back through rsinfo->setResult.
+	 * rsinfo->setDesc is set to the tuple description that we actually used
+	 * to build our tuples with, so the caller can verify we did what it was
+	 * expecting.
 	 */
 	return (Datum) 0;
 }
@@ -1159,9 +1147,15 @@ connectby_text_serial(PG_FUNCTION_ARGS)
 	MemoryContext oldcontext;
 
 	/* check to see if caller supports us returning a tuplestore */
-	if (!rsinfo || !(rsinfo->allowedModes & SFRM_Materialize))
-		elog(ERROR, "connectby: materialize mode required, but it is not "
-			 "allowed in this context");
+	if (rsinfo == NULL || !IsA(rsinfo, ReturnSetInfo))
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("set-valued function called in context that cannot accept a set")));
+	if (!(rsinfo->allowedModes & SFRM_Materialize))
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("materialize mode required, but it is not " \
+						"allowed in this context")));
 
 	if (fcinfo->nargs == 7)
 	{
@@ -1184,11 +1178,6 @@ connectby_text_serial(PG_FUNCTION_ARGS)
 	/* OK, use it then */
 	attinmeta = TupleDescGetAttInMetadata(tupdesc);
 
-	/* check to see if caller supports us returning a tuplestore */
-	if (!rsinfo->allowedModes & SFRM_Materialize)
-		elog(ERROR, "connectby requires Materialize mode, but it is not "
-			 "allowed in this context");
-
 	/* OK, go to work */
 	rsinfo->returnMode = SFRM_Materialize;
 	rsinfo->setResult = connectby(relname,
@@ -1208,10 +1197,10 @@ connectby_text_serial(PG_FUNCTION_ARGS)
 
 	/*
 	 * SFRM_Materialize mode expects us to return a NULL Datum. The actual
-	 * tuples are in our tuplestore and passed back through
-	 * rsinfo->setResult. rsinfo->setDesc is set to the tuple description
-	 * that we actually used to build our tuples with, so the caller can
-	 * verify we did what it was expecting.
+	 * tuples are in our tuplestore and passed back through rsinfo->setResult.
+	 * rsinfo->setDesc is set to the tuple description that we actually used
+	 * to build our tuples with, so the caller can verify we did what it was
+	 * expecting.
 	 */
 	return (Datum) 0;
 }
@@ -1408,16 +1397,16 @@ build_tuplestore_recursively(char *key_fld,
 		{
 			/*
 			 * Check that return tupdesc is compatible with the one we got
-			 * from the query, but only at level 0 -- no need to check
-			 * more than once
+			 * from the query, but only at level 0 -- no need to check more
+			 * than once
 			 */
 
 			if (!compatConnectbyTupleDescs(tupdesc, spi_tupdesc))
 				ereport(ERROR,
 						(errcode(ERRCODE_SYNTAX_ERROR),
 						 errmsg("invalid return type"),
-					 errdetail("Return and SQL tuple descriptions are " \
-							   "incompatible.")));
+						 errdetail("Return and SQL tuple descriptions are " \
+								   "incompatible.")));
 		}
 
 		for (i = 0; i < proc; i++)
@@ -1602,7 +1591,7 @@ compatConnectbyTupleDescs(TupleDesc ret_tupdesc, TupleDesc sql_tupdesc)
 				(errcode(ERRCODE_SYNTAX_ERROR),
 				 errmsg("invalid return type"),
 				 errdetail("SQL parent key field datatype does " \
-						"not match return parent key field datatype.")));
+						   "not match return parent key field datatype.")));
 
 	/* OK, the two tupdescs are compatible for our purposes */
 	return true;
@@ -1631,9 +1620,9 @@ compatCrosstabTupleDescs(TupleDesc ret_tupdesc, TupleDesc sql_tupdesc)
 						   "return rowid datatype.")));
 
 	/*
-	 * - attribute [1] of the sql tuple is the category; no need to check
-	 * it - attribute [2] of the sql tuple should match attributes [1] to
-	 * [natts] of the return tuple
+	 * - attribute [1] of the sql tuple is the category; no need to check it -
+	 * attribute [2] of the sql tuple should match attributes [1] to [natts]
+	 * of the return tuple
 	 */
 	sql_attr = sql_tupdesc->attrs[2];
 	for (i = 1; i < ret_tupdesc->natts; i++)
@@ -1648,52 +1637,6 @@ compatCrosstabTupleDescs(TupleDesc ret_tupdesc, TupleDesc sql_tupdesc)
 	return true;
 }
 
-static TupleDesc
-make_crosstab_tupledesc(TupleDesc spi_tupdesc, int num_categories)
-{
-	Form_pg_attribute sql_attr;
-	Oid			sql_atttypid;
-	TupleDesc	tupdesc;
-	int			natts;
-	AttrNumber	attnum;
-	char		attname[NAMEDATALEN];
-	int			i;
-
-	/*
-	 * We need to build a tuple description with one column for the
-	 * rowname, and num_categories columns for the values. Each must be of
-	 * the same type as the corresponding spi result input column.
-	 */
-	natts = num_categories + 1;
-	tupdesc = CreateTemplateTupleDesc(natts, false);
-
-	/* first the rowname column */
-	attnum = 1;
-
-	sql_attr = spi_tupdesc->attrs[0];
-	sql_atttypid = sql_attr->atttypid;
-
-	strcpy(attname, "rowname");
-
-	TupleDescInitEntry(tupdesc, attnum, attname, sql_atttypid,
-					   -1, 0);
-
-	/* now the category values columns */
-	sql_attr = spi_tupdesc->attrs[2];
-	sql_atttypid = sql_attr->atttypid;
-
-	for (i = 0; i < num_categories; i++)
-	{
-		attnum++;
-
-		sprintf(attname, "category_%d", i + 1);
-		TupleDescInitEntry(tupdesc, attnum, attname, sql_atttypid,
-						   -1, 0);
-	}
-
-	return tupdesc;
-}
-
 /*
  * Return a properly quoted literal value.
  * Uses quote_literal in quote.c
